/*
 * This file is part of the StrangeSearch project
 * Copyright 2001 Dan Dunham and Jeremy Davis
 *
 * It is distributed under the GNU Public Licence (GPL) version 2.  
 * See http://www.gnu.org/copyleft/gpl.html for further details 
 * of the GPL.
 */

#include "SsNuPinger.h"
#include <nb30.h>		//LINK with netapi32.lib

SsNuPinger::SsNuPinger()
{
	if(!NameStartUp())
	{
		ASSERT(0);
	}
}

SsNuPinger::~SsNuPinger()
{
	NameShutDown();
}

bool SsNuPinger::NameStartUp()
{
	int nRet;

	// Dynamically load the ICMP.DLL
	hndlIcmp = LoadLibrary("ICMP.DLL");
	if (hndlIcmp == NULL)
	{
		fprintf(stderr,"\nCould not load ICMP.DLL\n");
		return(false);
	}
	// Retrieve ICMP function pointers
	pIcmpCreateFile = (HANDLE (WINAPI *)(void))GetProcAddress(hndlIcmp,"IcmpCreateFile");
	pIcmpCloseHandle = (BOOL (WINAPI *)(HANDLE))GetProcAddress(hndlIcmp,"IcmpCloseHandle");
	pIcmpSendEcho = (DWORD (WINAPI *)(HANDLE,DWORD,LPVOID,WORD,PIPINFO,LPVOID,DWORD,DWORD))GetProcAddress(hndlIcmp,"IcmpSendEcho");

	// Check all the function pointers
	if (pIcmpCreateFile == NULL || pIcmpCloseHandle == NULL	|| pIcmpSendEcho == NULL)
	{
		fprintf(stderr,"\nError getting ICMP proc address\n");
		FreeLibrary(hndlIcmp);
		return(false);
	}

	// Init WinSock
	nRet = WSAStartup(0x0101, &wsaData );
	  if (nRet)
	  {
		fprintf(stderr,"\nWSAStartup() error: %d\n", nRet); 
		WSACleanup();
			FreeLibrary(hndlIcmp);
			return(false);
	  }

	  // Check WinSock version
	  if (0x0101 != wsaData.wVersion)
	  {
			fprintf(stderr,"\nWinSock version 1.1 not supported\n");
		WSACleanup();
			FreeLibrary(hndlIcmp);
		return(false);
	  }
	return(true);
}

void SsNuPinger::NameShutDown()
{
	WSACleanup();
	FreeLibrary(hndlIcmp);
}

bool SsNuPinger::bRetrieveName(char *Name, char *IP)
{
	LPHOSTENT pHost;
	in_addr    HostAddr;   
	HostAddr.s_addr = inet_addr(IP);
	pHost = gethostbyaddr((char *) &HostAddr.s_addr,sizeof(HostAddr.s_addr), PF_INET);
	if(pHost==NULL)
		return(false);
	strcpy(Name,pHost->h_name);
	return(true);
}

bool SsNuPinger::bRetrieveIP(char *Name, char *IP)
{

  LPHOSTENT pHost;			// Pointer to host entry structure
  int nResponses=0;


//remove any leading \\ from machine name (if any)
  char name2[256];
  int  y=0, z=0;
  memset(name2, 0, sizeof(name2));
  
  while(Name[y] != '\0')
  {
		if(Name[y]!='\\')
		{
			name2[z] = Name[y];
			z++;
			y++;	
		}
		else
			y++;
  }

	// Lookup destination
  pHost = gethostbyname(name2);

	if(pHost == NULL)
		return (false);

	// Copy the IP address
	//dwAddress = (DWORD *)(*pHost->h_addr_list);
  strcpy(IP, inet_ntoa((*(LPIN_ADDR)pHost->h_addr_list[0])));
	return(true);
}

bool SsNuPinger::bPingByIP(char *IP)
{
	LPHOSTENT pHost;				// Pointer to host entry structure
	in_addr		HostAddr;  
	DWORD		*dwAddress;			// IP Address
	HANDLE		hndlFile;			// Handle for IcmpCreateFile()
	IPINFO		ipInfo;				// IP Options structure
	DWORD		dwRet;				// DWORD return code
	struct in_addr iaDest;		// Internet address structure
	int			x,nResponses=0;

	HostAddr.s_addr = inet_addr(IP);
	if(HostAddr.s_addr == INADDR_NONE)      
		return(false);
	pHost = gethostbyaddr((char *) &HostAddr.s_addr,sizeof(HostAddr.s_addr), PF_INET);
	if(pHost==NULL)
		return(false);
	dwAddress = (DWORD *)(*pHost->h_addr_list);
	hndlFile = pIcmpCreateFile();
	for (x = 0; x < 4; x++)
	{
		// Set some reasonable default values
		ipInfo.Ttl = 255;
		ipInfo.Tos = 0;
		ipInfo.IPFlags = 0;
		ipInfo.OptSize = 0;
		ipInfo.Options = NULL;

		// Reqest an ICMP echo
		dwRet = pIcmpSendEcho(hndlFile, *dwAddress, NULL, 0, &ipInfo, &icmpEcho, sizeof(struct tagICMPECHO), 5000);

		iaDest.s_addr = icmpEcho.Source;

		if (icmpEcho.Status)
		{
			break;
		}
		if(dwRet > 0) // dwRet is number of ICMP packets returned to us
		{
		  nResponses++; 
		  break; // If we get any yes answers, the machine is on, stop checking
		}
	}
	pIcmpCloseHandle(hndlFile);
	if(nResponses > 0)
	{
		return(true); // machine is on right now, and IP is definately valid
	}
	return(false);
}
bool SsNuPinger::bPingByName(char *Name)
{
		HANDLE hndlFile;			// Handle for IcmpCreateFile()
  LPHOSTENT pHost;				// Pointer to host entry structure
  struct in_addr iaDest;		// Internet address structure
	DWORD *dwAddress;			// IP Address
	IPINFO ipInfo;				// IP Options structure
  int nResponses=0;
	DWORD dwRet;				// DWORD return code

  char name2[256];
  int  x,y=0, z=0;
  memset(name2, 0, sizeof(name2));
  
  while(Name[y] != '\0')
  {
    name2[z] = Name[y];
    z++;
    y++;
  }
  pHost = gethostbyname(name2);

	if(pHost == NULL)
		return(false);

	// Copy the IP address
	dwAddress = (DWORD *)(*pHost->h_addr_list);

	// Get an ICMP echo request handle        
	hndlFile = pIcmpCreateFile();
	for (x = 0; x < 4; x++)
	{
		// Set some reasonable default values
		ipInfo.Ttl = 255;
		ipInfo.Tos = 0;
		ipInfo.IPFlags = 0;
		ipInfo.OptSize = 0;
		ipInfo.Options = NULL;

		// Reqest an ICMP echo
		dwRet = pIcmpSendEcho(hndlFile, *dwAddress, NULL, 0, &ipInfo, &icmpEcho, sizeof(struct tagICMPECHO), 5000);

		iaDest.s_addr = icmpEcho.Source;

		if (icmpEcho.Status)
		{
			break;
		}
		if(dwRet > 0) // dwRet is number of ICMP packets returned to us
		{
		  nResponses++; 
		  break; // If we get any yes answers, the machine is on, stop checking
		}
	}

	// Close the echo request file handle
	pIcmpCloseHandle(hndlFile);
  if(nResponses > 0)
  {
    return(true); // machine is on right now, and IP is definately valid
  }
  return(false); // machine is down, but it has an IP

}

bool SsNuPinger::bGetMAC(unsigned char *MAC, char *IP)
{
		NCB		ncb;
	NBINFO  Adapter;
	UCHAR	uRetCode;
	char	Name2[16];
//NOTE: Netbios expects names to be 16 characters long (unused characters MUST=space)
//Netbios name does NOT include  "\\"
	if(IP==NULL)
		return(false);

	memset(Name2,32,sizeof(Name2));
	memcpy(Name2,IP,strlen(IP));
	Name2[15]=0;

    memset( &ncb, 0, sizeof(ncb) );
	memset(&Adapter,0,sizeof(NBINFO));
    ncb.ncb_command = NCBRESET;
    ncb.ncb_lana_num = 0;

    uRetCode = Netbios( &ncb );
	
	
    memset( &ncb, 0, sizeof(ncb) );
    ncb.ncb_command = NCBASTAT;
    ncb.ncb_lana_num = 0;
    //strcpy( (char*)ncb.ncb_callname,  "*               ");	//gets local
	strcpy((char*)ncb.ncb_callname,Name2);
    ncb.ncb_buffer = (unsigned char *) &Adapter;
    ncb.ncb_length = sizeof(Adapter);

    uRetCode = Netbios( &ncb );

    if ( uRetCode == 0 )
    {
		memcpy(MAC,Adapter.adapt.adapter_address,6);
		return(true);
    }	
	
	return(false);	
}

bool SsNuPinger::bGetNetbiosName(char *Name, char *IP)
{
	NCB		ncb;
	NBINFO  Adapter;
	UCHAR	uRetCode;
	char	Name2[16];
//NOTE: Netbios expects names to be 16 characters long (unused characters MUST=space)
//Netbios name does NOT include  "\\"
	if(Name==NULL)
		return(false);

	memset(Name2,32,sizeof(Name2));
	memcpy(Name2,IP,strlen(IP));
	Name2[15]=0;

    memset( &ncb, 0, sizeof(ncb) );
	memset(&Adapter,0,sizeof(NBINFO));
    ncb.ncb_command = NCBRESET;
    ncb.ncb_lana_num = 0;

    uRetCode = Netbios( &ncb );
	
	
    memset( &ncb, 0, sizeof(ncb) );
    ncb.ncb_command = NCBASTAT;
    ncb.ncb_lana_num = 0;
		strcpy((char*)ncb.ncb_callname,Name2);
    ncb.ncb_buffer = (unsigned char *) &Adapter;
    ncb.ncb_length = sizeof(Adapter);

    uRetCode = Netbios( &ncb );

    if ( uRetCode == 0 )
    {
			strncpy(Name,(char*)&Adapter.NameBuff[0],16);

			if(Name[14] == ' ')
			{
				int nLastCharIndex;
				for(nLastCharIndex = 14; nLastCharIndex >= 0; nLastCharIndex--)
				{
					if(Name[nLastCharIndex] != ' ')
					{
						Name[nLastCharIndex+1] = 0;
						break;
					}
				}
				if(nLastCharIndex == 0)
				{
					if(Name[0] == ' ')
					{
						Name[0] = 0;
					}
				}
			}
			else
			{
				Name[15] = 0;
			}

			return(true);
    }	
	
	return(false);
}



